module modelinput
  use class_particles, only: dp,aEW,MB,MC,MD,ME,MW,MZ,WW,WZ,MS,MT,MTA,MHL,MM,MU,pi,WT
  implicit none
  character(len=101) :: identifier,storeinput
  real(kind=dp)      :: Gf,MWos,MZos,WWos,WZos
  complex(kind=dp), parameter :: cone=1d0,cnul=0d0

  character(len=100) :: rs
  ! HS
  real(kind=dp) :: sa,l3,MHH
  ! THDM
  real(kind=dp) :: cab,tb,l5,MHA,MHC

  real(kind=dp) :: scalefac
  integer :: linc=0,printp=0,ytype=2

  contains

  subroutine read_modelinput

    integer :: i1,isep1,isep2
    integer, parameter :: linputline = 100
    character(len=100) :: inputline

    call init_default

50  read(*,'((a))',end=51,err=52) inputline


    if(inputline(1:1).eq.'!'.or.inputline(1:1).eq.'#') goto 50
    isep1=0
    isep2=linputline+1

    do i1=1,linputline
      if(inputline(i1:i1).eq.'=') then
        isep1=i1 
        goto 53
      elseif(inputline(i1:i1).eq.'!') then
        isep2=i1 
        goto 54
      endif
    enddo
53   continue

    do i1=isep1+1,linputline
      if(inputline(i1:i1).eq.'!') then
        isep2=i1 
        goto 54
      endif
    enddo
54   continue

    if (isep1 .gt. 0)then
      identifier= inputline(1:isep1-1)
      do i1=isep1-1,1,-1
        if (identifier(i1:i1).eq.' ') then
          identifier=identifier(1:i1-1)//identifier(i1+1:linputline)//' '
        elseif(identifier(i1:i1).ge.'A'.and.identifier(i1:i1).le.'Z')then
          call to_lower(identifier(i1:i1))
        end if
      end do
      write(storeinput,*) inputline(isep1+1:isep2-1)
      call read_identifier
      
      goto 50
    end if

    if (trim(inputline) .eq. '') goto 50
52   write(*,'(/(a))') ' Error message: read_input Stops here!'
    write(*,*) 'The following input line in the input'// &
               ' file is not appropriate:  ',inputline
    stop 9


51  continue

    call convert_input_sm
    call set_input_sm

  end subroutine read_modelinput

  subroutine to_lower(str)
    character(*), intent(in out) :: str
    integer :: i

    do i = 1, len(str)
      select case(str(i:i))
        case("A":"Z")
          str(i:i) = achar(iachar(str(i:i))+32)
      end select
    end do  
  end subroutine to_lower


  subroutine init_default()
  
    MT = 173.34d0
    MZos = 91.1876d0
    MWos = 80.385d0
    WZos = 2.4952d0
    WWos = 2.0850d0
    Gf =  0.116637d-4


    MHL = 125d0
    ME = 0d0
    MM = 0d0
    MTA = 0d0
    MU = 0d0
    MD = 0d0
    MS = 0d0
    MC = 0d0
    MB = 0d0
    
  end subroutine init_default

  subroutine read_identifier

    if(identifier.eq.'gf') then
      read(storeinput,'(f50.50)',err=61) Gf
      read(storeinput,*) Gf

    elseif(identifier.eq.'mw') then
      read(storeinput,'(f50.50)',err=61) MWos
      read(storeinput,*) MWos

    elseif(identifier.eq.'gw') then
      read(storeinput,'(f50.50)',err=61) WWos
      read(storeinput,*) WWos

    elseif(identifier.eq.'mz') then
      read(storeinput,'(f50.50)',err=61) MZos
      read(storeinput,*) MZos

    elseif(identifier.eq.'gz') then
      read(storeinput,'(f50.50)',err=61) WZos
      read(storeinput,*) WZos

    elseif(identifier.eq.'mhl') then
      read(storeinput,'(f50.50)',err=61) MHL
      read(storeinput,*) MHL

    elseif(identifier.eq.'mt') then
      read(storeinput,'(f50.50)',err=61) MT
      read(storeinput,*) MT

    elseif(identifier.eq.'mb') then
      read(storeinput,'(f50.50)',err=61) MB
      read(storeinput,*) MB

    elseif(identifier.eq.'mc') then
      read(storeinput,'(f50.50)',err=61) MC
      read(storeinput,*) MC

    elseif(identifier.eq.'mixrs') then
      read(storeinput,'(A16)',err=61) rs
      read(storeinput,*) rs

    else if(identifier.eq.'linc') then
      read(storeinput,'(i100)',err=61) linc
      read(storeinput,*) linc

    else if(identifier.eq.'printp') then
      read(storeinput,'(i100)',err=61) printp
      read(storeinput,*) printp

    else if(identifier.eq.'ytype') then
      read(storeinput,'(i100)',err=61) ytype
      read(storeinput,*) ytype

    else if(identifier.eq.'l3') then
      read(storeinput,'(f50.50)',err=61) l3
      read(storeinput,*) l3

    else if(identifier.eq.'sa') then
      read(storeinput,'(f50.50)',err=61) sa
      read(storeinput,*) sa

    else if(identifier.eq.'cab') then
      read(storeinput,'(f50.50)',err=61) cab
      read(storeinput,*) cab

    else if(identifier.eq.'tb') then
      read(storeinput,'(f50.50)',err=61) tb
      read(storeinput,*) tb

    else if(identifier.eq.'l5') then
      read(storeinput,'(f50.50)',err=61) l5
      read(storeinput,*) l5

    else if(identifier.eq.'mhh') then
      read(storeinput,'(f50.50)',err=61) mhh
      read(storeinput,*) mhh

    else if(identifier.eq.'mha') then
      read(storeinput,'(f50.50)',err=61) mha
      read(storeinput,*) mha

    else if(identifier.eq.'mhc') then
      read(storeinput,'(f50.50)',err=61) mhc
      read(storeinput,*) mhc

    else if(identifier.eq.'scalefac') then
      read(storeinput,'(f50.50)',err=61) scalefac
      read(storeinput,*) scalefac
        
    else if(identifier .ne. ' ') then
      write(*,'(/(a))') ' Error message: read_input stops here!'
      write(*,*) 'The following Identifier in the input'// &
                  ' file is not known:  ',identifier
       stop 9
     end if

    goto 62

61  write(*,'(/(a))') ' Error message: read_input Stops here!'
    write(*,*) 'In the input file the input is not '// &
               'appropriate for identifier ''', &
               trim(identifier),''' : ',storeinput
    stop 9

62  continue
      
  end subroutine read_identifier

  subroutine convert_input_sm

    MW = MWos/sqrt(1d0+(WWos/MWos)**2)
    WW = WWos/sqrt(1d0+(WWos/MWos)**2)

    MZ = MZos/sqrt(1d0+(WZos/MZos)**2)
    WZ = WZos/sqrt(1d0+(WZos/MZos)**2)

    aEW = Gf/pi*MW**2d0*(1d0-MW**2d0/MZ**2d0)* &
          sqrt(2d0)*cone

  end subroutine convert_input_sm

  subroutine set_input_sm
    use modelfile, only: set_parameter_mdl
    call set_parameter_mdl("aEW",aEW)
    call set_parameter_mdl("aS",0.1*cone)

    call set_parameter_mdl("MHL",MHL*cone)
    call set_parameter_mdl("WHL",cnul)
    call set_parameter_mdl("MHH",400d0*cone)
    call set_parameter_mdl("WHH",cnul)

    call set_parameter_mdl("MTA",MTA*cone)
    call set_parameter_mdl("WTA",cnul)
    call set_parameter_mdl("MM",MM*cone)
    call set_parameter_mdl("WM",cnul)
    call set_parameter_mdl("ME",ME*cone)

    call set_parameter_mdl("MT",MT*cone)
    call set_parameter_mdl("WT",cnul)
    call set_parameter_mdl("MB",MB*cone)
    call set_parameter_mdl("WB",cnul)
    call set_parameter_mdl("MC",MC*cone)
    call set_parameter_mdl("WC",cnul)
    call set_parameter_mdl("MS",MS*cone)
    call set_parameter_mdl("MU",MU*cone)
    call set_parameter_mdl("MD",MD*cone)
  end subroutine set_input_sm


end module modelinput
